@@ -9,6 +9,8 @@
 #' @param id Character. The variable in `data` that identifies the observational
 #' cluster unit. Not necessary when `data` is a list object of simulated data generated
 #' with `mlts_sim`.
+#' @param group Character. The variable in `data` that identifies the grouping variable
+#' across cluster units.
 #' @param ts Character. The variable(s) in `data` that contain the time-series
 #' construct(s) or their indicator variable(s). If multiple constructs are provided
 #' in the `model`, multiple entries are necessary. Note that the order of variable
@@ -32,6 +34,7 @@
 #' will be grand-mean centered before model fitting by default. Set `center_covs` to `FALSE`
 #' when including categorical predictors into the set of `covariates`. Note that in this case,
 #' additional continuous covariates should be grand-mean centered prior to using `mlts_fit`.
+#' If `group` is specified, covariates will be centered on their respective group mean.
 #' @param time Character. The variable in `data` that contains the (continuous) time of observation.
 #' @param tinterval The step interval for approximating equally spaced observations in time by
 #' insertion of missing values, to be specified with respect to the time stamp variable
@@ -45,6 +48,9 @@
 #' for overnight lags (see Details).
 #' @param n_overnight_NAs Optional. The number of `NA` rows to add after the last
 #' observation of each day (if `days` is provided).
+#' @param max_NA_seq Integer. Specify a maximum number of consecutive missing values.
+#' Can decrease estimation times drastically in the presence of very long sequences of missing values
+#' (e.g., when setting tinterval to values of small time steps).
 #' @param na.rm logical. Per default, missing values remain in the data and
 #' will be imputed during model estimation. Set to `TRUE` to remove all rows with
 #' missing values in variables given in `ts`.
@@ -56,6 +62,7 @@
 #' The default is 2 (see \code{\link[rstan]{stan}}).
 #' @param monitor_person_pars Logical. Should person parameters (i.e., values of the
 #' latent variables) be stored? Default is FALSE.
+#' @param monitor_all_pars Logical. Should all parameters be stored? Default is FALSE.
 #' @param get_SD_latent Logical. Set to `TRUE` to obtain standardized estimates
 #' in multiple-indicator models.
 #' @param print_message Logical. Print messages based on defined inputs (default = TRUE).
@@ -107,6 +114,7 @@
 mlts_fit <- function(model,
                      data =NULL,
                      id,
+                     group=NULL,
                      ts,
                      covariates = NULL,
                      outcomes = NULL,
@@ -117,11 +125,13 @@ mlts_fit <- function(model,
                      beep = NULL,
                      days = NULL,
                      n_overnight_NAs,
+                     max_NA_seq = NULL,
                      na.rm = FALSE,
                      iter = 500,
                      chains = 2,
                      cores = 2,
                      monitor_person_pars = FALSE,
+                     monitor_all_pars = FALSE,
                      get_SD_latent = FALSE,
                      fit_model = TRUE,
                      print_message = TRUE,
@@ -153,15 +163,17 @@ mlts_fit <- function(model,
       }))
     )
   }
-  cat(call_inds)
+
+  if(print_message == TRUE){cat(call_inds)}
 
   # simulated data used
   data.simulated = ifelse(class(data)[1] == "mlts_simdata", TRUE, FALSE)
 
   # check if data is class "mlts_simdata"
   if(data.simulated == TRUE) {
-    message("Simulated data provided:",
-    "\nTrue scores used in the data generation will be added to the returned object.")
+
+    if(print_message == TRUE){message("Simulated data provided:",
+    "\nTrue scores used in the data generation will be added to the returned object.")}
 
     par_labels <- merge(x = par_labels, data$model[,c("Param", "true.val")],
                        by = "Param", sort = FALSE)
@@ -250,7 +262,7 @@ mlts_fit <- function(model,
   btw_vars = c(names(covariates), names(outcomes), names(outcome_pred_btw))
   if(length(btw_vars)>0){
     if(sum(is.na(data[,btw_vars]))>0){
-      stop("No missing values on between-level variables are allowed.")
+      stop("Missing values on between-level variables are not allowed.")
     }
   }
 
@@ -261,7 +273,9 @@ mlts_fit <- function(model,
                       days = days, n_overnight_NAs = n_overnight_NAs,
                       na.rm = na.rm, covariates = covariates,
                       outcomes = outcomes,
-                      outcome_pred_btw = outcome_pred_btw)
+                      outcome_pred_btw = outcome_pred_btw,
+                      max_NA_seq = max_NA_seq,
+                      group = group)
 
   # ======================================================================
 
@@ -270,7 +284,46 @@ mlts_fit <- function(model,
   isLatent <- ifelse(sum(model$Model == "Measurement")>0,TRUE,FALSE)
 
 
-  # BY MODEL TYPE ========================================================
+  # PRIOR CHECKS =========================================================
+  ## grand-means of ts variables
+
+  ts_means = unlist(lapply(ts, function(x){mean(data[,x], na.rm=TRUE)}))
+
+  ## naive comparison against priors for fixed effects of mean levels
+  if(isLatent == FALSE){
+    ts_means = ts_means[infos$is_wcen == 1]
+    prior_distr = infos$prior_gamma[1:length(ts_means),]
+    prior_distr$ts_means = ts_means
+    # obtain p-values under prior distribution
+    ps = unlist(lapply(1:length(ts_means), FUN = function(x){
+      stats::pnorm(q    = prior_distr$ts_means[x],
+                   mean = prior_distr$prior_location[x],
+                   sd   = prior_distr$prior_scale[x])
+    }))
+    fail = any(ps < .10) | any(1-ps < .10)
+
+    if(fail == TRUE){
+      fail_text = c("\nCurrent prior specification appears to not match the observed grand-means of ts-variables. Consider adjusting the priors or rescaling ts-variables to a smaller range (e.g., -3 to 3).")
+      warning(fail_text)
+    }
+
+
+  } else {
+    ts_means = ts_means[infos$p_is_wcen == 1]
+    fail = any(abs(ts_means) > 10)
+
+    if(fail == TRUE){
+      fail_text = c("\nObserved grand-means of ts-variables > 10 (or < -10) detected. Make sure that priors are adjusted accordingly or consider rescaling ts-variables to a smaller range (e.g., -3 to 3).")
+      message(fail_text)
+    }
+
+  }
+
+
+
+  # ======================================================================
+
+  # ESTIMATION BY MODEL TYPE ==================================================
   # VAR(1) Models -------------------------------------------------------------
 
   ## Single-indicator VAR(1) model
@@ -279,7 +332,7 @@ mlts_fit <- function(model,
     standata <- VARprepare(model = model, data = data, ts = ts,
                           covariates = covariates, outcomes = outcomes,
                           outcome_pred_btw = outcome_pred_btw,
-                          center_covs = center_covs)
+                          center_covs = center_covs, group = group)
 
     # model fit
     pars <- c("gammas","b_fix", "sigma", "sd_R", "bcorr",
@@ -287,6 +340,9 @@ mlts_fit <- function(model,
     if(monitor_person_pars == TRUE){
       pars = c(pars, "b_free")
     }
+    if(monitor_all_pars == TRUE){
+      pars <- NA
+    }
 
     if(standata$n_inno_cors == 0){
       if(fit_model==T){
@@ -326,7 +382,7 @@ mlts_fit <- function(model,
     standata <- VARprepare(model = model, data = data, ts = ts,
                            covariates = covariates, outcomes = outcomes,
                            outcome_pred_btw = outcome_pred_btw,
-                           center_covs = center_covs)
+                           center_covs = center_covs, group = group)
     # latent variable SDs requested?
     standata$standardized = ifelse(get_SD_latent == TRUE, 1, 0)
     if(get_SD_latent == FALSE & print_message == TRUE){
@@ -382,7 +438,16 @@ mlts_fit <- function(model,
     # add posteriors with adapted names
     posteriors <- rstan::extract(stanfit, inc_warmup = FALSE, permuted = FALSE,
                              pars = par_labels$Param_stan)
-    dimnames(posteriors)$parameters <- par_labels$Param
+
+    # names of parameters
+    par_names <- par_labels$Param
+    # adjust parameter names for multiple groups
+    if(standata$G > 1){
+      g_prefix <- sapply(par_labels$group, function(x){standata$group_lab[x]})
+      par_names <- paste0(g_prefix,": ",par_names)
+    }
+
+    dimnames(posteriors)$parameters <- par_names
 
     # create a summary table using the monitor-function in rstan
     sums <- rstan::monitor(stanfit, print = FALSE)
@@ -400,10 +465,18 @@ mlts_fit <- function(model,
     if(monitor_person_pars == TRUE){
       sums.i = sums[startsWith(sums$Param_stan, "b_free"),1:ncol(sums)]
 
+    } else {
+      sums.i = rstan::get_posterior_mean(stanfit)
+      sums.i = sums.i[startsWith(row.names(sums.i), "b_free"),]
+      sums.i = data.frame(
+        "Param_stan" = rownames(sums.i),
+        "mean" = sums.i[,dim(sums.i)[2]]
+        )
+    }
+
       # extract infos
       pars <- gsub(sums.i$Param_stan, pattern = "b_free[", replacement = "", fixed = TRUE)
-      pars <- gsub(pars, pattern = "]", replacement = "", fixed = TRUE
-                   )
+      pars <- gsub(pars, pattern = "]", replacement = "", fixed = TRUE)
       ID_new <- sapply(pars, function(x){strsplit(x,split = ",")[[1]][1]})
       pars <- sapply(pars, function(x){as.integer(strsplit(x,split = ",")[[1]][2])})
       pars <- sapply(pars, function(x){infos$re_pars$Param[x]})
@@ -412,6 +485,7 @@ mlts_fit <- function(model,
       sums.i$Param_stan <- NULL
       sums.i = cbind(
         "num_id" = ID_new,
+        "group" = sapply(as.numeric(ID_new),function(x){standata$g_id[x]}),
         "Param" = pars,
         sums.i
       )
@@ -431,14 +505,8 @@ mlts_fit <- function(model,
       id.match = unique(data[c(id, "num_id")])
       sums.i = merge(id.match, y = sums.i, by = "num_id", all = TRUE)
 
-
-
     } else {
-        # we could think about adding the posterior means here
-        sums.i <- NA
-    }
 
-  } else {
     posteriors <- NA
     pop.sums <- NA
     sums.i <- NA
